@@ -9,7 +9,7 @@ This module was written by Matthias Cuntz while at Institut National de
 Recherche pour l'Agriculture, l'Alimentation et l'Environnement (INRAE), Nancy,
 France.
 
-:copyright: Copyright 2020-2021 Matthias Cuntz - mc (at) macu (dot) de
+:copyright: Copyright 2020- Matthias Cuntz - mc (at) macu (dot) de
 :license: MIT License, see LICENSE for details.
 
 .. moduleauthor:: Matthias Cuntz
@@ -24,45 +24,81 @@ The following functions are provided:
    format_coord_map
    format_coord_scatter
    get_slice
+   get_standard_name
+   get_units
    list_intersection
+   parse_entry
+   selvar
    set_axis_label
    set_miss
    spinbox_values
    vardim2var
+   xzip_dim_name_length
    zip_dim_name_length
 
 History
-    * Written Nov-Dec 2020 by Matthias Cuntz (mc (at) macu (dot) de)
-    * General get_slice function from individual methods for x, y, y2, z,
-      Dec 2020, Matthias Cuntz
-    * Added arithmetics to apply on axis/dimensions such as mean, std, etc.,
-      Dec 2020, Matthias Cuntz
-    * Added clone_ncvmain, removing its own module, Dec 2020, Matthias Cuntz
-    * added SEPCHAR and DIMMETHODS, Jan 2021, Matthias Cuntz
-    * pass only ncvMain widget to clone_ncvmain, Jan 2021, Matthias Cuntz
-    * pass only root widget to clone_ncvmain, Jan 2021, Matthias Cuntz
-    * set correct missing value for date variable in numpy's datetime64[ms]
-      format May 2021, Matthias Cuntz
-    * added format_coord functions for scatter, contour, and map,
-      May 2021, Matthias Cuntz
-    * replaced add_cyclic_point with add_cyclic as submitted to cartopy,
-      Jun 2021, Matthias Cuntz
-    * removed SEPCHAR, Jun 2021, Matthias Cuntz
+   * Written Nov-Dec 2020 by Matthias Cuntz (mc (at) macu (dot) de)
+   * General get_slice function from individual methods for x, y, y2, z,
+     Dec 2020, Matthias Cuntz
+   * Added arithmetics to apply on axis/dimensions such as mean, std, etc.,
+     Dec 2020, Matthias Cuntz
+   * Added clone_ncvmain, removing its own module, Dec 2020, Matthias Cuntz
+   * Added SEPCHAR and DIMMETHODS, Jan 2021, Matthias Cuntz
+   * Pass only ncvMain widget to clone_ncvmain, Jan 2021, Matthias Cuntz
+   * Pass only root widget to clone_ncvmain, Jan 2021, Matthias Cuntz
+   * Set correct missing value for date variable in numpy's datetime64[ms]
+     format May 2021, Matthias Cuntz
+   * Added format_coord functions for scatter, contour, and map,
+     May 2021, Matthias Cuntz
+   * Replaced add_cyclic_point with add_cyclic as submitted to cartopy,
+     Jun 2021, Matthias Cuntz
+   * Removed SEPCHAR, Jun 2021, Matthias Cuntz
+   * Final add_cyclic, has_cyclic from cartopy v0.20.1,
+     Nov 2021, Matthias Cuntz
+   * Address fi.variables[name] directly by fi[name], Jan 2024, Matthias Cuntz
+   * Add selvar to allow multiple netcdf files, Jan 2024, Matthias Cuntz
+   * Remove [ms] from check for datetime in format_coord on axes,
+     Oct 2024, Matthias Cuntz
+   * Use CustomTkinter in clone_ncvmain if installed, Nov 2024, Matthias Cuntz
+   * Increased digits in format_coord_scatter, Jan 2025, Matthias Cuntz
+   * Increased digits in format_coord_contour and format_coord_map,
+     Jan 2025, Matthias Cuntz
+   * Use f-string in zip_dim_name_length, Feb 2025, Matthias Cuntz
+   * Added xzip_dim_name_length, get_standard_name, get_units,
+     Feb 2025, Matthias Cuntz
+   * Allow xarray in selvar, Feb 2025, Matthias Cuntz
+   * Add parse_entry from dfvutils, Jun 2025, Matthias Cuntz
+   * Use ncvScreen for window sizes, Nov 2025, Matthias Cuntz
+   * Copy parse_entry from dfvutils again, deducing datetime string,
+     Nov 2025, Matthias Cuntz
 
 """
-from __future__ import absolute_import, division, print_function
 import tkinter as tk
+try:
+    from customtkinter import CTkToplevel as Toplevel
+except ModuleNotFoundError:
+    from tkinter import Toplevel
+from math import isfinite
 import numpy as np
 import matplotlib.dates as mpld
 import cartopy.crs as ccrs
+try:
+    import xarray as xr
+    ihavex = True
+except ModuleNotFoundError:
+    ihavex = False
+from .ncvscreen import ncvScreen
+import ncvue
 
 
 __all__ = ['DIMMETHODS',
-           'add_cyclic', 'clone_ncvmain',
+           'add_cyclic', 'has_cyclic', 'clone_ncvmain',
            'format_coord_contour', 'format_coord_map', 'format_coord_scatter',
-           'get_slice',
-           'list_intersection', 'set_axis_label', 'set_miss',
-           'spinbox_values', 'vardim2var', 'zip_dim_name_length']
+           'get_slice', 'get_standard_name', 'get_units',
+           'list_intersection', 'parse_entry',
+           'selvar', 'set_axis_label', 'set_miss',
+           'spinbox_values', 'vardim2var',
+           'xzip_dim_name_length', 'zip_dim_name_length']
 
 
 DIMMETHODS = ('mean', 'std', 'min', 'max', 'ptp', 'sum', 'median', 'var')
@@ -89,7 +125,7 @@ def _add_cyclic_data(data, axis=-1):
     ----------
     data : ndarray
         An n-dimensional array of data to add a cyclic point to.
-    axis: int, optional
+    axis : int, optional
         Specifies the axis of the data array to add the cyclic point to.
         Defaults to the right-most axis.
 
@@ -102,64 +138,64 @@ def _add_cyclic_data(data, axis=-1):
     try:
         slicer[axis] = slice(0, 1)
     except IndexError:
-        estr = 'The specified axis does not correspond to an array dimension.'
-        raise ValueError(estr)
+        raise ValueError(
+            'The specified axis does not correspond to an array dimension.')
     npc = np.ma if np.ma.is_masked(data) else np
     return npc.concatenate((data, data[tuple(slicer)]), axis=axis)
 
 
-def _add_cyclic_lon(lon, axis=-1, cyclic=360):
+def _add_cyclic_x(x, axis=-1, cyclic=360):
     """
-    Add a cyclic point to a longitude array.
+    Add a cyclic point to a x/longitude coordinate array.
 
     Parameters
     ----------
-    lon: ndarray, optional
-        An array which specifies the coordinate values for
+    x : ndarray
+        An array which specifies the x-coordinate values for
         the dimension the cyclic point is to be added to.
-    axis: int, optional
-        Specifies the axis of the longitude array to add the cyclic point to.
-        Defaults to the right-most axis.
-    cyclic: float, optional
+    axis : int, optional
+        Specifies the axis of the x-coordinate array to add the cyclic point
+        to. Defaults to the right-most axis.
+    cyclic : float, optional
         Width of periodic domain (default: 360)
 
     Returns
     -------
-    The coordinate `lon` with a cyclic point added.
+    The coordinate array ``x`` with a cyclic point added.
 
     """
-    npc = np.ma if np.ma.is_masked(lon) else np
-    # get cyclic longitudes
-    # clon is the code from basemap (addcyclic)
+    npc = np.ma if np.ma.is_masked(x) else np
+    # get cyclic x-coordinates
+    # cx is the code from basemap (addcyclic)
     # https://github.com/matplotlib/basemap/blob/master/lib/mpl_toolkits/basemap/__init__.py
-    clon = (np.take(lon, [0], axis=axis) +
-            cyclic * np.sign(np.diff(np.take(lon, [0, -1], axis=axis),
-                                     axis=axis)))
+    cx = (np.take(x, [0], axis=axis) +
+          cyclic * np.sign(np.diff(np.take(x, [0, -1], axis=axis),
+                                   axis=axis)))
     # basemap ensures that the values do not exceed cyclic
     # (next code line). We do not do this to deal with rotated grids that
     # might have values not exactly 0.
-    #     clon = npc.where(clon <= cyclic, clon, np.mod(clon, cyclic))
-    return npc.concatenate((lon, clon), axis=axis)
+    #     cx = npc.where(cx <= cyclic, cx, np.mod(cx, cyclic))
+    return npc.concatenate((x, cx), axis=axis)
 
 
-def _has_cyclic(lon, axis=-1, cyclic=360, prec=1e-4):
+def has_cyclic(x, axis=-1, cyclic=360, precision=1e-4):
     """
-    Check if longitudes already have a cyclic point.
+    Check if x/longitude coordinates already have a cyclic point.
 
     Checks all differences between the first and last
-    longitudes along `axis` to be less than `prec`.
+    x-coordinates along ``axis`` to be less than ``precision``.
 
     Parameters
     ----------
-    lon: ndarray, optional
-        An array with the coordinate values to be checked for cyclic points.
-    axis: int, optional
-        Specifies the axis of the `lon` array to be checked.
+    x : ndarray
+        An array with the x-coordinate values to be checked for cyclic points.
+    axis : int, optional
+        Specifies the axis of the ``x`` array to be checked.
         Defaults to the right-most axis.
-    cyclic: float, optional
+    cyclic : float, optional
         Width of periodic domain (default: 360).
-    prec: float, optional
-        Maximal difference between first and last longitude to detect
+    precision : float, optional
+        Maximal difference between first and last x-coordinate to detect
         cyclic point (default: 1e-4).
 
     Returns
@@ -167,145 +203,161 @@ def _has_cyclic(lon, axis=-1, cyclic=360, prec=1e-4):
     True if a cyclic point was detected along the given axis,
     False otherwise.
 
+    Examples
+    --------
+    Check for cyclic x-coordinate in one dimension.
+    >>> import numpy as np
+    >>> lons = np.arange(0, 360, 60)
+    >>> clons = np.arange(0, 361, 60)
+    >>> print(has_cyclic(lons))
+    False
+    >>> print(has_cyclic(clons))
+    True
+
+    Check for cyclic x-coordinate in two dimensions.
+    >>> lats = np.arange(-90, 90, 30)
+    >>> lon2d, lat2d = np.meshgrid(lons, lats)
+    >>> clon2d, clat2d = np.meshgrid(clons, lats)
+    >>> print(has_cyclic(lon2d))
+    False
+    >>> print(has_cyclic(clon2d))
+    True
+
     """
-    npc = np.ma if np.ma.is_masked(lon) else np
+    npc = np.ma if np.ma.is_masked(x) else np
     # transform to 0-cyclic, assuming e.g. -180 to 180 if any < 0
-    lon1 = np.mod(npc.where(lon < 0, lon + cyclic, lon), cyclic)
-    dd = np.diff(np.take(lon1, [0, -1], axis=axis), axis=axis)
-    if npc.all(np.abs(dd) < prec):
+    x1 = np.mod(npc.where(x < 0, x + cyclic, x), cyclic)
+    dd = np.diff(np.take(x1, [0, -1], axis=axis), axis=axis)
+    if npc.all(np.abs(dd) < precision):
         return True
     else:
         return False
 
 
-def add_cyclic(data, coord=None, rowcoord=None, axis=-1,
-               cyclic=360, prec=1e-4):
+def add_cyclic(data, x=None, y=None, axis=-1,
+               cyclic=360, precision=1e-4):
     """
     Add a cyclic point to an array and optionally corresponding
-    column (`coord` ~ longitudes) and row coordinates
-    (`rowcoord` ~ latitudes).
-
-    The call is `add_cyclic(data[, coord[, rowcoord]])`.
+    x/longitude and y/latitude coordinates.
 
     Checks all differences between the first and last
-    coordinates along `axis` to be less than `prec`.
+    x-coordinates along ``axis`` to be less than ``precision``.
 
     Parameters
     ----------
     data : ndarray
         An n-dimensional array of data to add a cyclic point to.
-    coord: ndarray, optional
-        An n-dimensional array which specifies the coordinate values
+    x : ndarray, optional
+        An n-dimensional array which specifies the x-coordinate values
         for the dimension the cyclic point is to be added to, i.e. normally the
         longitudes. Defaults to None.
 
-        If `coord` is given than add_cyclic checks if a cyclic point is
+        If ``x`` is given then *add_cyclic* checks if a cyclic point is
         already present by checking all differences between the first and last
-        coordinates to be less than `prec`.
+        coordinates to be less than ``precision``.
         No point is added if a cyclic point was detected.
 
-        `coord.shape[-1]` must be `data.shape[axis]` if coord is 1- or
-        2-dimensional,
-        `coord.shape[axis]` must be `data.shape[axis]` otherwise.
-    rowcoord: ndarray, optional
-        An n-dimensional array with the variable of the row
-        coordinate, i.e. normally the latitudes.
+        If ``x`` is 1- or 2-dimensional, ``x.shape[-1]`` must equal
+        ``data.shape[axis]``, otherwise ``x.shape[axis]`` must equal
+        ``data.shape[axis]``.
+    y : ndarray, optional
+        An n-dimensional array with the values of the y-coordinate, i.e.
+        normally the latitudes.
         The cyclic point simply copies the last column. Defaults to None.
 
-        `rowcoord.shape[-1]` must be `data.shape[axis]` if rowcoord is 1- or
-        2-dimensional,
-        `rowcoord.shape[axis]` must be `data.shape[axis]` otherwise.
-    axis: int, optional
-        Specifies the axis of the data array to add the cyclic point to,
-        i.e. normally the longitudes. Defaults to the right-most axis.
-    cyclic: int or float, optional
+        No cyclic point is added if ``y`` is 1-dimensional.
+        If ``y`` is 2-dimensional, ``y.shape[-1]`` must equal
+        ``data.shape[axis]``, otherwise ``y.shape[axis]`` must equal
+        ``data.shape[axis]``.
+    axis : int, optional
+        Specifies the axis of the arrays to add the cyclic point to,
+        i.e. axis with changing x-coordinates. Defaults to the right-most axis.
+    cyclic : int or float, optional
         Width of periodic domain (default: 360).
-    prec: float, optional
-        Maximal difference between first and last coordinate to detect
+    precision : float, optional
+        Maximal difference between first and last x-coordinate to detect
         cyclic point (default: 1e-4).
 
     Returns
     -------
     cyclic_data
         The data array with a cyclic point added.
-    cyclic_coord
-        The coordinate with a cyclic point, only returned if the `coord`
+    cyclic_x
+        The x-coordinate with a cyclic point, only returned if the ``x``
+        keyword was supplied.
+    cyclic_y
+        The y-coordinate with the last column of the cyclic axis duplicated,
+        only returned if ``x`` was 2- or n-dimensional and the ``y``
         keyword was supplied.
-    cyclic_rowcoord
-        The row coordinate with the last column duplicated, only returned
-        if `coord` was 2- or n-dimensional and the `rowcoord` keyword was
-        supplied.
 
     Examples
     --------
     Adding a cyclic point to a data array, where the cyclic dimension is
     the right-most dimension.
-    .. testsetup::
-        >>> from distutils.version import LooseVersion
-        >>> import numpy as np
-        >>> if LooseVersion(np.__version__) >= '1.14.0':
-        ...     # To provide consistent doctests.
-        ...     np.set_printoptions(legacy='1.13')
+
     >>> import numpy as np
     >>> data = np.ones([5, 6]) * np.arange(6)
     >>> cyclic_data = add_cyclic(data)
     >>> print(cyclic_data)  # doctest: +NORMALIZE_WHITESPACE
-    [[ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]]
+    [[0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]]
+
+    Adding a cyclic point to a data array and an associated x-coordinate.
 
-    Adding a cyclic point to a data array and an associated coordinate.
     >>> lons = np.arange(0, 360, 60)
-    >>> cyclic_data, cyclic_lons = add_cyclic(data, coord=lons)
+    >>> cyclic_data, cyclic_lons = add_cyclic(data, x=lons)
     >>> print(cyclic_data)  # doctest: +NORMALIZE_WHITESPACE
-    [[ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]]
+    [[0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]]
     >>> print(cyclic_lons)
-    [   0.   60.  120.  180.  240.  300.  360.]
+    [  0  60 120 180 240 300 360]
 
     Adding a cyclic point to a data array and an associated 2-dimensional
-    coordinate.
+    x-coordinate.
+
     >>> lons = np.arange(0, 360, 60)
     >>> lats = np.arange(-90, 90, 180/5)
     >>> lon2d, lat2d = np.meshgrid(lons, lats)
-    >>> cyclic_data, cyclic_lon2d = add_cyclic(data, coord=lon2d)
+    >>> cyclic_data, cyclic_lon2d = add_cyclic(data, x=lon2d)
     >>> print(cyclic_data)  # doctest: +NORMALIZE_WHITESPACE
-    [[ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]]
+    [[0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]]
     >>> print(cyclic_lon2d)
-    [[   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]]
+    [[  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]]
+
+    Adding a cyclic point to a data array and the associated 2-dimensional
+    x- and y-coordinates.
 
-    Adding a cyclic point to a data array and an associated 2-dimensional
-    coordinate and a second raw variable.
     >>> lons = np.arange(0, 360, 60)
     >>> lats = np.arange(-90, 90, 180/5)
     >>> lon2d, lat2d = np.meshgrid(lons, lats)
     >>> cyclic_data, cyclic_lon2d, cyclic_lat2d = add_cyclic(
-    ...     data, coord=lon2d, rowcoord=lat2d)
+    ...     data, x=lon2d, y=lat2d)
     >>> print(cyclic_data)  # doctest: +NORMALIZE_WHITESPACE
-    [[ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]
-     [ 0. 1. 2. 3. 4. 5. 0.]]
+    [[0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]
+     [0. 1. 2. 3. 4. 5. 0.]]
     >>> print(cyclic_lon2d)
-    [[   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]
-     [   0.   60.  120.  180.  240.  300.  360.]]
+    [[  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]
+     [  0  60 120 180 240 300 360]]
     >>> print(cyclic_lat2d)
     [[-90. -90. -90. -90. -90. -90. -90.]
      [-54. -54. -54. -54. -54. -54. -54.]
@@ -313,18 +365,19 @@ def add_cyclic(data, coord=None, rowcoord=None, axis=-1,
      [ 18.  18.  18.  18.  18.  18.  18.]
      [ 54.  54.  54.  54.  54.  54.  54.]]
 
-    Not adding a cyclic point if cyclic point detected in coord.
+    Not adding a cyclic point if cyclic point detected in x.
+
     >>> lons = np.arange(0, 361, 72)
     >>> lats = np.arange(-90, 90, 180/5)
     >>> lon2d, lat2d = np.meshgrid(lons, lats)
     >>> cyclic_data, cyclic_lon2d, cyclic_lat2d = add_cyclic(
-    ...     data, coord=lon2d, rowcoord=lat2d)
+    ...     data, x=lon2d, y=lat2d)
     >>> print(cyclic_data)  # doctest: +NORMALIZE_WHITESPACE
-    [[ 0. 1. 2. 3. 4. 5.]
-     [ 0. 1. 2. 3. 4. 5.]
-     [ 0. 1. 2. 3. 4. 5.]
-     [ 0. 1. 2. 3. 4. 5.]
-     [ 0. 1. 2. 3. 4. 5.]]
+    [[0. 1. 2. 3. 4. 5.]
+     [0. 1. 2. 3. 4. 5.]
+     [0. 1. 2. 3. 4. 5.]
+     [0. 1. 2. 3. 4. 5.]
+     [0. 1. 2. 3. 4. 5.]]
     >>> print(cyclic_lon2d)
     [[  0  72 144 216 288 360]
      [  0  72 144 216 288 360]
@@ -339,45 +392,44 @@ def add_cyclic(data, coord=None, rowcoord=None, axis=-1,
      [ 54.  54.  54.  54.  54.  54.]]
 
     """
-    if coord is None:
+    if x is None:
         return _add_cyclic_data(data, axis=axis)
+    # if x was given
+    if x.ndim > 2:
+        xaxis = axis
     else:
-        if (coord.ndim > 2):
-            caxis = axis
-        else:
-            caxis = -1
-        if coord.shape[caxis] != data.shape[axis]:
-            estr = 'coord.shape[{}] does not match'.format(caxis)
-            estr += ' the size of the corresponding dimension of'
-            estr += ' the data array: coord.shape[{}] ='.format(caxis)
-            estr += ' {}, data.shape[{}] = {}.'.format(
-                coord.shape[caxis], axis, data.shape[axis])
-            raise ValueError(estr)
-        if _has_cyclic(coord, axis=caxis, cyclic=cyclic, prec=prec):
-            if rowcoord is None:
-                return data, coord
-            else:
-                return data, coord, rowcoord
-        else:
-            odata = _add_cyclic_data(data, axis=axis)
-            ocoord = _add_cyclic_lon(coord, axis=caxis, cyclic=cyclic)
-            if rowcoord is None:
-                return odata, ocoord
-            else:
-                if (rowcoord.ndim > 2):
-                    raxis = axis
-                else:
-                    raxis = -1
-                if rowcoord.shape[raxis] != data.shape[axis]:
-                    estr = 'rowcoord.shape[{}] does not match'.format(raxis)
-                    estr += ' the size of the corresponding dimension of'
-                    estr += ' the data array: rowcoord.shape[{}] ='.format(
-                        raxis)
-                    estr += ' {}, data.shape[{}] = {}.'.format(
-                        rowcoord.shape[raxis], axis, data.shape[axis])
-                    raise ValueError(estr)
-                orowcoord = _add_cyclic_data(rowcoord, axis=raxis)
-                return odata, ocoord, orowcoord
+        xaxis = -1
+    if x.shape[xaxis] != data.shape[axis]:
+        estr = (f'x.shape[{xaxis}] does not match the size of the'
+                f' corresponding dimension of the data array:'
+                f' x.shape[{xaxis}] = {x.shape[xaxis]},'
+                f' data.shape[{axis}] = {data.shape[axis]}.')
+        raise ValueError(estr)
+    if has_cyclic(x, axis=xaxis, cyclic=cyclic, precision=precision):
+        if y is None:
+            return data, x
+        # if y was given
+        return data, x, y
+    # if not has_cyclic, add cyclic points to data and x
+    out_data = _add_cyclic_data(data, axis=axis)
+    out_x = _add_cyclic_x(x, axis=xaxis, cyclic=cyclic)
+    if y is None:
+        return out_data, out_x
+    # if y was given
+    if y.ndim == 1:
+        return out_data, out_x, y
+    if y.ndim > 2:
+        yaxis = axis
+    else:
+        yaxis = -1
+    if y.shape[yaxis] != data.shape[axis]:
+        estr = (f'y.shape[{yaxis}] does not match the size of the'
+                f' corresponding dimension of the data array:'
+                f' y.shape[{yaxis}] = {y.shape[yaxis]},'
+                f' data.shape[{axis}] = {data.shape[axis]}.')
+        raise ValueError(estr)
+    out_y = _add_cyclic_data(y, axis=yaxis)
+    return out_data, out_x, out_y
 
 
 def clone_ncvmain(widget):
@@ -407,19 +459,27 @@ def clone_ncvmain(widget):
         import sys
         sys.exit()
 
-    root = tk.Toplevel()
+    root = Toplevel()
     root.name = 'ncvClone'
     root.title("Secondary ncvue window")
-    root.geometry('1000x800+150+100')
+    # root.geometry('1000x800+150+100')
 
     root.top = widget.top
+    screen = ncvScreen(root.top)
+    root.geometry(screen.secondwin)
 
     # https://stackoverflow.com/questions/46505982/is-there-a-way-to-clone-a-tkinter-widget
     cls = widget.__class__
     clone = cls(root)
-    for key in widget.configure():
-        if key != 'class':
-            clone.configure({key: widget.cget(key)})
+    try:
+        for key in widget.configure():
+            if key != 'class':
+                clone.configure({key: widget.cget(key)})
+    except TypeError:
+        # in case of CustomTkinter
+        cls = ncvue.ncvMain
+        clone = cls(root)
+        clone.pack(fill=tk.BOTH, expand=1)
 
     return clone
 
@@ -455,16 +515,18 @@ def format_coord_contour(x, y, ax, xx, yy, zz):
         xarr = xx[0, :]
     else:
         xarr = xx
-    if xx.dtype == np.dtype('<M8[ms]'):
+    # if xx.dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(xx.dtype, np.datetime64):
         xarr = mpld.date2num(xarr)
     if yy.ndim > 1:
         yarr = yy[:, 0]
     else:
         yarr = yy
-    if yy.dtype == np.dtype('<M8[ms]'):
+    # if yy.dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(yy.dtype, np.datetime64):
         yarr = mpld.date2num(yarr)
-    if ((x > xarr.min()) & (x <= xarr.max()) &
-        (y > yarr.min()) & (y <= yarr.max())):
+    if ( (x > xarr.min()) & (x <= xarr.max()) &
+         (y > yarr.min()) & (y <= yarr.max()) ):
         col = np.searchsorted(xarr, x) - 1
         row = np.searchsorted(yarr, y) - 1
         xout = xarr[col]
@@ -473,26 +535,31 @@ def format_coord_contour(x, y, ax, xx, yy, zz):
     else:
         xout = x
         yout = y
-        if zz.dtype == np.dtype('<M8[ms]'):
+        # if zz.dtype.type == np.dtype('datetime64').type:
+        if np.issubdtype(zz.dtype, np.datetime64):
             zout = np.datetime64('NaT')
         else:
             zout = np.nan
 
     # Special treatment for datetime
     # https://stackoverflow.com/questions/49267011/matplotlib-datetime-from-event-coordinates
-    if xx.dtype == np.dtype('<M8[ms]'):
+    # if xx.dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(xx.dtype, np.datetime64):
         xstr = mpld.num2date(xout).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        xstr  = '{:.4g}'.format(xout)
-    if yy.dtype == np.dtype('<M8[ms]'):
+        xstr = '{:.6g}'.format(xout)
+    # if yy.dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(yy.dtype, np.datetime64):
         ystr = mpld.num2date(yout).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        ystr  = '{:.4g}'.format(yout)
-    if zz.dtype == np.dtype('<M8[ms]'):
+        ystr = '{:.6g}'.format(yout)
+    # if zz.dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(zz.dtype, np.datetime64):
         zstr = mpld.num2date(zout).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        zstr = '{:.4g}'.format(zout)
-    out = 'x=' + xstr + ', y=' + ystr + ', z=' + zstr
+        zstr = '{:.6g}'.format(zout)
+    # out = 'x=' + xstr + ', y=' + ystr + ', z=' + zstr
+    out = f'x={xstr}, y={ystr}, z={zstr}'
     return out
 
 
@@ -540,13 +607,14 @@ def format_coord_map(x, y, ax, xx, yy, zz):
     # ystr  = '{:.4g}'.format(yout)
     xstr  = '{:.4g}'.format(xout)
     ystr  = '{:.4g}'.format(yout)
-    zstr  = '{:.4g}'.format(zout)
+    zstr  = '{:.6g}'.format(zout)
     ns = 'N' if lat >= 0. else 'S'
     ew = 'E' if lon >= 0. else 'W'
     latstr = u'{:.4f} \u00b0{:s}'.format(abs(lat), ns)
     lonstr = u'{:.4f} \u00b0{:s}'.format(abs(lon), ew)
-    out  = u'x=' + xstr + ', y=' + ystr + ' (' + lonstr + ', ' + latstr + ')'
-    out += ' z=' + zstr
+    # out  = u'x=' + xstr + ', y=' + ystr + ' (' + lonstr + ', ' + latstr + ')'
+    # out += ' z=' + zstr
+    out = f'x={xstr}, y={ystr} ({lonstr}, {latstr}), z={zstr}'
     return out
 
 
@@ -588,20 +656,22 @@ def format_coord_scatter(x, y, ax, ax2, xdtype, ydtype, y2dtype):
 
     # Special treatment for datetime
     # https://stackoverflow.com/questions/49267011/matplotlib-datetime-from-event-coordinates
-    if xdtype == np.dtype('<M8[ms]'):
+    # if xdtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(xdtype, np.datetime64):
         xstr = mpld.num2date(x).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        xstr  = '{:.3g}'.format(x)
-    if ydtype == np.dtype('<M8[ms]'):
+        xstr  = '{:.6g}'.format(x)
+    # if ydtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(ydtype, np.datetime64):
         ystr = mpld.num2date(ax_coord[1]).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        ystr  = '{:.3g}'.format(ax_coord[1])
-    if y2dtype == np.dtype('<M8[ms]'):
+        ystr  = '{:.6g}'.format(ax_coord[1])
+    # if y2dtype.type == np.dtype('datetime64').type:
+    if np.issubdtype(y2dtype, np.datetime64):
         y2str = mpld.num2date(y).strftime('%Y-%m-%d %H:%M:%S')
     else:
-        y2str = '{:.3g}'.format(y)
-    out  = 'Left: (' + xstr + ', ' + ystr + ')'
-    out += ' Right: (' + xstr + ', ' + y2str + ')'
+        y2str = '{:.6g}'.format(y)
+    out = f'Left: ({xstr}, {ystr}) Right: ({xstr}, {y2str})'
     return out
 
 
@@ -619,17 +689,21 @@ def get_slice(dimspins, y):
     Returns
     -------
     ndarray
-        Slice of `y` chosen by with spinboxes.
+        Slice of `y` chosen with spinboxes.
 
     Examples
     --------
-    >>> vy = vardim2var(y)
-    >>> yy = self.fi.variables[vy]
+    >>> gy, vy = vardim2var(y, self.groups)
+    >>> yy = selvar(self, vy)
     >>> miss = get_miss(self, yy)
-    >>> yy = get_slice_y(self.yd, yy).squeeze()
+    >>> yy = get_slice(self.yd, yy).squeeze()
     >>> yy = set_miss(miss, yy)
 
     """
+    isxarray = False
+    if ihavex:
+        if isinstance(y, xr.DataArray):
+            isxarray = True
     methods = ['all']
     methods.extend(DIMMETHODS)
     dd = []
@@ -640,13 +714,17 @@ def get_slice(dimspins, y):
             s = slice(0, y.shape[i])
         else:
             idim = int(dim)
-            s = slice(idim, idim+1)
+            s = slice(idim, idim + 1)
         dd.append(dim)
         ss.append(s)
     if len(ss) > 0:
         imeth = list_intersection(dd, DIMMETHODS)
         if len(imeth) > 0:
-            yout = y[tuple(ss)]
+            if isxarray:
+                sargs = dict(zip(y.dims, ss))
+                yout = y.isel(**sargs).to_numpy()
+            else:
+                yout = y[tuple(ss)]
             ii = [ i for i, d in enumerate(dd) if d in imeth ]
             ii.reverse()  # last axis first
             for i in ii:
@@ -668,11 +746,68 @@ def get_slice(dimspins, y):
                     yout = np.ma.var(yout, axis=i)
             return yout
         else:
-            return y[tuple(ss)]
+            if isxarray:
+                sargs = dict(zip(y.dims, ss))
+                yout = y.isel(**sargs).to_numpy()
+            else:
+                yout = y[tuple(ss)]
+            return yout
     else:
         return np.array([], dtype=y.dtype)
 
 
+def get_standard_name(ivar):
+    """
+    Get standard_name, long_name, or name attribute of netcdf variable
+
+    Parameters
+    ----------
+    ivar : netCDF4._netCDF4.Variable
+        netcdf variable
+
+    Returns
+    -------
+    string
+        standard_name, long_name, or name of variable `ivar`
+
+    Examples
+    --------
+    >>> name = get_standard_name(self.fi['Tair])
+
+    """
+    if hasattr(ivar, 'standard_name'):
+        return ivar.standard_name
+    elif hasattr(ivar, 'long_name'):
+        return ivar.long_name
+    else:
+        return ivar.name
+
+
+def get_units(ivar):
+    """
+    Get units attribute of netcdf variable
+
+    Parameters
+    ----------
+    ivar : netCDF4._netCDF4.Variable
+        netcdf variable
+
+    Returns
+    -------
+    string
+        units or empty string
+
+    Examples
+    --------
+    >>> units = get_units(self.fi['Tair])
+
+    """
+    if hasattr(ivar, 'units'):
+        return ivar.units
+    else:
+        return ''
+
+
 def list_intersection(lst1, lst2):
     """
     Intersection of two lists.
@@ -706,6 +841,127 @@ def list_intersection(lst1, lst2):
         return [ ll for ll in lst1 if ll in lst2 ]
 
 
+def parse_entry(text):
+    """
+    Convert text string to correct data type
+
+    Parse an entry field to None, bool, int, float, datetime, list, dict
+
+    Parameters
+    ----------
+    text : str
+        String from entry field
+
+    Returns
+    -------
+    None, bool, int, float, datetime, list, dict
+
+    Examples
+    --------
+    >>> parse_entry('7')
+    7
+    >>> parse_entry('7,3')
+    [7, 3]
+
+    """
+    if ',' in text:
+        # # list or str
+        # try:
+        #     tt = eval(f'[{text}]')
+        # except SyntaxError:
+        #     tt = text
+        # parse each element
+        stext = text.split(',')
+        tt = [ parse_entry(ss) for ss in stext ]
+    elif text == 'None':
+        # None
+        tt = None
+    elif text == 'True':
+        # bool True
+        tt = True
+    elif text == 'False':
+        # bool False
+        tt = False
+    elif ':' in text:
+        # dict, datetime, or str
+        try:
+            tt = eval(f'{{{text}}}')
+        except SyntaxError:
+            try:
+                tt = np.datetime64(text)
+            except ValueError:
+                tt = text
+    elif text.count('-') == 2:
+        # datetime or str
+        try:
+            tt = np.datetime64(text)
+        except ValueError:
+            tt = text
+    else:
+        tt = text
+
+    # if above gave str, check for scalars
+    if tt == text:
+        try:
+            # int
+            tt = int(text)
+        except ValueError:
+            try:
+                # float
+                tt = float(text)
+            except ValueError:
+                # str
+                tt = text
+            try:
+                if not isfinite(tt):
+                    # keep NaN and Inf string
+                    tt = text
+            except TypeError:
+                pass
+    return tt
+
+
+def selvar(self, var):
+    """
+    Extract variable from correct file.
+
+    Parameters
+    ----------
+    self : class
+        ncvue Tk class having fi and groups
+    var : string
+        Variable name including group or file identifier,
+        such as 'Qle', 'group1/Qle', or 'file0001/Qle'
+
+    Returns
+    -------
+    netcdf variable
+        Pointer to variable in file
+
+    Examples
+    --------
+    selvar(self, var)
+
+    """
+    if len(self.fi) == 0:
+        return
+    elif self.usex:
+        return self.fi[var]
+    elif len(self.fi) == 1:
+        fil = self.fi[0]
+        return fil[var]
+    else:
+        if '/' in var:
+            f = var[0:var.find('/')].rstrip()
+            idxf = list(self.groups).index(f)
+            vv = var[var.find('/') + 1:].rstrip()
+            fil = self.fi[idxf]
+            return fil[vv]
+        else:
+            fil = self.fi[0]
+            return fil[var]
+
+
 def set_axis_label(ncvar):
     """
     Set label plotting axis from name and unit of given variable
@@ -723,7 +979,7 @@ def set_axis_label(ncvar):
 
     Examples
     --------
-    >>> ylab = set_axis_label(fi.variables['w_soil'])
+    >>> ylab = set_axis_label(fi['w_soil'])
 
     """
     try:
@@ -759,12 +1015,12 @@ def set_miss(miss, x):
 
     Examples
     --------
-    >>> x = fi.variables['time']
+    >>> x = fi['time']
     >>> miss = get_miss(self, x)
     >>> x = set_miss(miss, x)
 
     """
-    if x.dtype == np.dtype('<M8[ms]'):
+    if np.issubdtype(x.dtype, np.datetime64):
         default = np.datetime64('NaT')
     else:
         default = np.nan
@@ -801,27 +1057,64 @@ def spinbox_values(ndim):
         return (0,)
 
 
-def vardim2var(vardim):
+def vardim2var(vardim, groups=[]):
     """
-    Extract variable name from 'variable (dim1=ndim1,)' string.
+    Extract group index and variable name from 'group/variable (dim1=ndim1,)'.
 
     Parameters
     ----------
     vardim : string
-        Variable name with dimensions, such as 'latitude (lat=32,lon=64)'.
+        Variable name with dimensions, such as 'latitude (lat=32,lon=64)'
+        or with group 'file1/latitude (lat=32,lon=64)'
 
     Returns
     -------
-    string
-        Variable name.
+    int, string
+        group index, variable name
 
     Examples
     --------
     >>> vardim2var('latitude (lat=32,lon=64)')
-    latitude
+    0 latitude
 
     """
-    return vardim[0:vardim.rfind('(')].rstrip()
+    var = vardim[0:vardim.rfind('(')].rstrip()
+    if '/' in var:
+        g = var[0:var.find('/')].rstrip()
+        ig = list(groups).index(g)
+    else:
+        ig = 0
+    return ig, var
+
+
+def xzip_dim_name_length(ncvar):
+    """
+    Combines dimension names and length of xarray variable in list of strings.
+
+    Parameters
+    ----------
+    ncvar : xarray.DataArray
+        xarray variable
+
+    Returns
+    -------
+    list
+        List of dimension name and length in the form 'dim=len'.
+
+    Examples
+    --------
+    >>> import xarray as xr
+    >>> ifile = 'test.nc'
+    >>> fi = xr.open_dataset(ifile)
+    >>> w_soil = fi['w_soil']
+    >>> print(zip_dim_name_length(w_soil))
+    ('ntime=17520', 'nsoil=30')
+
+    """
+    out = []
+    for i in range(ncvar.ndim):
+        out.append(f'{ncvar.dims[i]}={ncvar.shape[i]}')
+    return out
 
 
 def zip_dim_name_length(ncvar):
@@ -843,12 +1136,12 @@ def zip_dim_name_length(ncvar):
     >>> import netCDF4 as nc
     >>> ifile = 'test.nc'
     >>> fi = nc.Dataset(ifile, 'r')
-    >>> w_soil = fi.variables['w_soil']
+    >>> w_soil = fi['w_soil']
     >>> print(zip_dim_name_length(w_soil))
     ('ntime=17520', 'nsoil=30')
 
     """
     out = []
     for i in range(ncvar.ndim):
-        out.append(ncvar.dimensions[i]+'='+str(ncvar.shape[i]))
+        out.append(f'{ncvar.dimensions[i]}={ncvar.shape[i]}')
     return out
